package ru.dbotthepony.kstarbound.client.render

import org.lwjgl.opengl.GL46.*
import ru.dbotthepony.kstarbound.client.ClientChunk
import ru.dbotthepony.kstarbound.client.gl.GLStateTracker
import ru.dbotthepony.kstarbound.client.gl.VertexTransformers
import ru.dbotthepony.kstarbound.math.Matrix4fStack
import ru.dbotthepony.kstarbound.math.Vector2d
import ru.dbotthepony.kstarbound.world.entities.Entity
import ru.dbotthepony.kstarbound.world.entities.Projectile
import java.io.Closeable

/**
 * Базовый класс, отвечающий за отрисовку определённого ентити в мире
 *
 * Считается, что процесс отрисовки ограничен лишь одним слоем (т.е. отрисовка происходит в один проход)
 */
open class EntityRenderer(val state: GLStateTracker, val entity: Entity, open var chunk: ClientChunk?) : Closeable {
	open val renderPos: Vector2d get() = entity.pos

	open fun render(stack: Matrix4fStack) {

	}

	open fun renderDebug() {
		if (chunk?.world?.client?.settings?.debugCollisions == true) {
			state.quadWireframe(entity.movement.worldAABB)
		}
	}

	open val layer: Int = 100

	override fun close() {

	}

	companion object {
		fun getRender(state: GLStateTracker, entity: Entity, chunk: ClientChunk? = null): EntityRenderer {
			return when (entity) {
				is Projectile -> ProjectileRenderer(state, entity, chunk)
				else -> EntityRenderer(state, entity, chunk)
			}
		}
	}
}

open class ProjectileRenderer(state: GLStateTracker, entity: Projectile, chunk: ClientChunk?) : EntityRenderer(state, entity, chunk) {
	private val def = entity.def
	private val texture = state.loadNamedTextureSafe(def.image.texture)
	private val animator = FrameSetAnimator(def.image, def.animationCycle, true)

	init {
		texture.textureMagFilter = GL_NEAREST
	}

	override fun render(stack: Matrix4fStack) {
		state.shaderVertexTexture.use()
		state.shaderVertexTexture.transform.set(stack.last)
		state.activeTexture = 0
		state.shaderVertexTexture["_texture"] = 0
		texture.bind()

		animator.advance()

		val stateful = state.flat2DTexturedQuads.statefulSmall
		val builder = stateful.builder

		builder.begin()

		val (u0, v0) = texture.pixelToUV(def.image.frames[animator.frame].texturePosition)
		val (u1, v1) = texture.pixelToUV(def.image.frames[animator.frame].textureEndPosition)

		builder.quadZ(0f, 0f, 1f, def.image.frames[animator.frame].aspectRatioHW, 5f, VertexTransformers.uv(u0, v0, u1, v1))

		stateful.upload()
		stateful.draw()
	}
}
